/*-
 * Copyright (c) 2000 Mitsuru IWASAKI <iwasaki@jp.freebsd.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$FreeBSD$
 */

#include <sys/file.h>
#include <sys/ioctl.h>

#include <dev/power/powerio.h>
#include <dev/power/powervar.h>

#include <err.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "libpower.h"

#define POWERDEV	"/dev/power"

int 
power_get_syspm_info(struct power_syspm_info *infp)
{
	int	 fd, error;
	union	 power_info_args args;

	fd = open(POWERDEV, O_RDONLY);
	if (fd == -1) {
		warn("can't open %s", POWERDEV);
		return (-1);
	}

	args.battery_unit = -1;
	error = ioctl(fd, POWERIO_GET_INFO, &args);
	if (error == -1) {
		warn("ioctl");
	} else {
		infp->pm_type = args.power_info.pm_type;
		infp->acline = args.power_info.acline;
		infp->batteries = args.power_info.batteries;
		infp->batt_stat = args.power_info.batt_stat;
		infp->batt_life = args.power_info.batt_life;
		infp->batt_time = args.power_info.batt_time;
	}

	close(fd);
	return (error);
}

int 
power_get_batt_info(u_int unit, struct power_batt_info *infp)
{
	int	 fd, error;
	union	 power_info_args args;

	fd = open(POWERDEV, O_RDONLY);
	if (fd == -1) {
		warn("can't open %s", POWERDEV);
		return (-1);
	}

	args.battery_unit = unit;
	error = ioctl(fd, POWERIO_GET_INFO, &args);
	if (error == -1) {
		warn("ioctl");
	} else {
		infp->batt_stat = args.power_info.batt_stat;
		infp->batt_flag = args.power_info.batt_flag;
		infp->batt_life = args.power_info.batt_life;
		infp->batt_time = args.power_info.batt_time;
	}

	close(fd);
	return (error);
}

static int
power_set_sleep_state(int state)
{
	int	 fd, error;

	if (state < POWER_SLEEP_STATE_STANDBY ||
	    state > POWER_SLEEP_STATE_HIBERNATE) {
		warn("invalid sleep state");
		return (-1);
	}

	fd = open(POWERDEV, O_RDONLY);
	if (fd == -1) {
		warn("can't open %s", POWERDEV);
		return (-1);
	}

	error = ioctl(fd, POWERIO_SET_SEEP, &state);
	if (error == -1) {
		warn("ioctl");
	}

	close(fd);
	return (error);
}

int
power_standby(void)
{
	int	 error;

	error = power_set_sleep_state(POWER_SLEEP_STATE_STANDBY);
	return (error);
}

int
power_suspend(void)
{
	int	 error;

	error = power_set_sleep_state(POWER_SLEEP_STATE_SUSPEND);
	return (error);
}

int
power_hibernate(void)
{
	int	 error;

	error = power_set_sleep_state(POWER_SLEEP_STATE_HIBERNATE);
	return (error);
}

