/*-
 * Copyright (c) 2000 Mitsuru IWASAKI <iwasaki@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/conf.h>

#include <dev/power/powerio.h>
#include <dev/power/powervar.h>

static d_open_t		 power_open;
static d_ioctl_t	 power_ioctl;

#define POWER_CDEV_MAJOR  210

static struct cdevsw power_cdevsw = {
	power_open,
	nullclose,
	noread,
	nowrite,
	power_ioctl,
	nopoll,
	nommap,
	nostrategy,
	"power",
	POWER_CDEV_MAJOR,
	nodump,
	nopsize,
	0,
	-1
};

static u_int	 power_pm_type = POWER_PM_TYPE_UNKNOWN;	/* APM or ACPI */
static power_pm_fn_t	power_pm_fn;
static void	*power_pm_arg = NULL;

void
power_pm_register(u_int pm_type, power_pm_fn_t pm_fn, void *pm_arg)
{

	if (power_pm_type == POWER_PM_TYPE_UNKNOWN) {
		power_pm_type = pm_type;
		power_pm_fn = pm_fn;
		power_pm_arg = pm_arg;
	}
}

static void
power_identify(driver_t *driver, device_t parent)
{
	device_t	 child;

	child = BUS_ADD_CHILD(parent, 0, "power", 0);
	if (child == NULL)
		panic("power_identify");
}

static int
power_probe(device_t dev)
{

	/* only one instance always */
	return (device_get_unit(dev));
}

static int
power_attach(device_t dev)
{

	make_dev(&power_cdevsw, 0, 0, 5, 0664, "power");
	return (0);
}

static device_method_t power_methods[] = {
	/* Device interface */
	DEVMETHOD(device_identify,	power_identify),
	DEVMETHOD(device_probe,		power_probe),
	DEVMETHOD(device_attach,	power_attach),

	{0, 0}
};

static driver_t power_driver = {
	"power",
	power_methods,
	1,		/* no softc */
};

static devclass_t	 power_devclass;
DRIVER_MODULE(power, nexus, power_driver, power_devclass, 0, 0);

static int
power_open(dev_t dev, int flag, int fmt, struct proc * p)
{

	if (power_pm_type == POWER_PM_TYPE_UNKNOWN) {
		return (ENXIO);
	}

	return (0);
}

static int
power_ioctl(dev_t dev, u_long cmd, caddr_t addr, int flag, struct proc *p)
{

	if (power_pm_type == POWER_PM_TYPE_UNKNOWN) {
		return (ENXIO);
	}

	return (power_pm_fn(cmd, addr, power_pm_arg));
}

