/*-
 * Copyright (c) 2000 Mitsuru IWASAKI <iwasaki@jp.freebsd.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$FreeBSD$
 */

#ifndef _POWERVAR_H_
#define _POWERVAR_H_

/* Power management system type */
#define POWER_PM_TYPE_APM		0x00
#define POWER_PM_TYPE_ACPI		0x01
#define POWER_PM_TYPE_UNKNOWN		0xff

/* AC line status */
#define POWER_ACLINE_STATUS_OFF		0x00
#define POWER_ACLINE_STATUS_ON		0x01
#define POWER_ACLINE_STATUS_UNKNOWN	0xff

/* Batteries */
#define POWER_BATTERIES_UNKNOWN		(u_int)-1

/* Battery status */
#define POWER_BATT_STATUS_HIGH		0x00
#define POWER_BATT_STATUS_LOW		0x01
#define POWER_BATT_STATUS_CRITICAL	0x02
#define POWER_BATT_STATUS_CHARGING	0x03
#define POWER_BATT_STATUS_UNKNOWN	0xff

/* Battery life */
#define POWER_BATT_LIFE_UNKNOWN		0xff

/* Battery time */
#define POWER_BATT_TIME_UNKNOWN		-1

/* Battery flag (more detailed battery status per unit) */
#define POWER_BATT_FLAG_HIGH		0x01
#define POWER_BATT_FLAG_LOW		0x02
#define POWER_BATT_FLAG_CRITICAL	0x04
#define POWER_BATT_FLAG_CHARGING	0x08
#define POWER_BATT_FLAG_NOT_PRESENT	0x10
#define POWER_BATT_FLAG_NO_SYSTEM	0x80
#define POWER_BATT_FLAG_UNKNOWN		0xff

/* Sleep state */
#define POWER_SLEEP_STATE_STANDBY	0x00
#define POWER_SLEEP_STATE_SUSPEND	0x01
#define POWER_SLEEP_STATE_HIBERNATE	0x02

#ifdef _KERNEL
typedef int (*power_pm_fn_t)(u_long, caddr_t, void*);
extern void	 power_pm_register(u_int, power_pm_fn_t, void *);
#endif

#endif	/* !_POWERVAR_H_ */

